/*****************************************************************************/
/*!	\file			IEC61850API.h 
 *	\brief 		IEC61850 API header file
 *	\par 		SystemCORP Embedded Technology Pty Ltd
 *
 *				Suite 4/12, Brodie Hall Drive,
 *				Technology Park,
 *				Bentley,
 *				Perth, WA 6102,
 *				Australia.
 *
 *				Phone	: +61 (8) 9472 8500 
 *				Fax		: +61 (8) 9472 9500
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

/******************************************************************************
*	Defines
******************************************************************************/

#ifndef IEC61850_INCLUDED
#define IEC61850_INCLUDED 1

#include "sysctype.h"
#include "IEC61850Types.h"

/*!	\brief PIS10 Version Number */
#define PIS10_VERSION         "1.36.07.008"

/*
Library Control complier options

In a windows environment 

	if PIS10_STATIC is defined then the library will be complied for static linking

	For making a DLL define PIS10_EXPORTS
	For using the DLL no NOT define PIS10_EXPORTS
*/

#ifdef WIN32
	#ifdef PIS10_STATIC
		#define PIS10_API
	#else
		#if defined(PIS10_EXPORTS) || defined(DLL_EXPORT)
			#define PIS10_API __declspec(dllexport)
		#else
			#define PIS10_API __declspec(dllimport)
		#endif
	#endif
#else
	/*!API function prefix  */
	#define PIS10_API
#endif

/*! 

\defgroup 	DataAttributeAccess 		Data Attribute Access functions
\defgroup 	Management 				IEC61850 Object managment functions
\defgroup 	Time 					Time functions
\defgroup 	Support 					Support functions
\defgroup 	Control 					Control functions

 */


#ifdef __cplusplus
extern "C" {
#endif
/*! \brief 	Create a client or server object with call-backs for reading, writing and updating data objects
	\ingroup 	Management

	\param[in] 	ptParameters  	IEC 61850 Object Parameters
	\param[out] 	peErrorCode  		pointer to return integer for error code if an error occured
	
	\return 	Pointer to a new IEC 61850 object
	\return 	NULL if an error occured (errorCode will contain an error code)

	\see IEC61850_Free()

	Server example usage:
	\code
		// Create Server
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		IEC61850 myServer = NULL;
		struct IEC61850_Parameters	tServerParam;
		
		tServerParam.ClientServerFlag 	= IEC61850_SERVER;		// This is a IEC 61850 Server
		tServerParam.uiOptions			= IEC61850_OPTION_NONE;	// No options set
		tClientParam.uiCmdTermTimeOut	= 0;						// Command Termination timeout 0 will default to 10 seconds
		tServerParam.ptReadCallback 	= ReadFunction;			// Assign Read Callback Function
		tServerParam.ptWriteCallback 	= WriteFunction;			// Assign Write Callback Function
		tServerParam.ptUpdateCallback 	= NULL;					// No Update Callback Function for Server
		tServerParam.ptSelectCallback	= NULL;					// Ignoring Select commands
		tServerParam.ptOperateCallback	= NULL;					// Ignoring Operate commands
		tServerParam.ptCancelCallback 	= NULL;					// Ignoring Cancel commands
		
		
		//Create a server
		myServer = IEC61850_Create(&tServerParam, &error);
		if(myServer == NULL)
		{
			printf("Server Failed to create: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode


	Client example usage:
	\code

		// Create Client
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		IEC61850 myClient = NULL;
		struct IEC61850_Parameters	tClientParam;
		
		
		tClientParam.ClientServerFlag	= IEC61850_CLIENT;			// This is a IEC 61850 Client	
		tClientParam.uiOptions			= 0;						// Reserved for future use (set to ZERO) 
		tClientParam.uiCmdTermTimeOut	= 0;						// Command Termination timeout 0 will default to 10 seconds
		tClientParam.ptReadCallback 	= NULL;						// No Read Callback Function for Client
		tClientParam.ptWriteCallback	= NULL;						// No Update Callback Function for Client
		tClientParam.ptUpdateCallback 	= UpdateFunction;			// Assign Update Callback Function for Client
		tClientParam.ptSelectCallback	= NULL;					// Ignoring Select commands
		tClientParam.ptOperateCallback	= NULL;					// Ignoring Operate commands
		tClientParam.ptCancelCallback 	= NULL;					// Ignoring Cancel commands
		
		//Create a client
		myClient = IEC61850_Create(&tClientParam, &error);
		if(myClient == NULL)
		{
			printf("Client Failed to create: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode
 */
PIS10_API IEC61850 IEC61850_Create(struct IEC61850_Parameters * ptParameters, enum IEC61850_ErrorCodes * peErrorCode);

/*!  \brief 		Free memory used by IEC 61850 object.
	\ingroup 	Management

	 \param[in] 	clientServerObject	  		Server/Client object to free

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	\see IEC61850_Create()

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		
		error = IEC61850_Free(myServer);  //Frees myServer
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Free IEC61850 Object has failed: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode  	 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_Free(IEC61850 clientServerObject);

/*!  \brief 	Load a SCL file into the IEC 61850
	\ingroup 	Management

	 \param[in] 	clientServer	client/Server object
	 \param[in] 	SCLFileName		File name of the SCL file
	 
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		
		error = IEC61850_LoadSCLFile(myServer,"myIED.cid"); // Load in IED configuration SCL file
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Loading SCL file has failed: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode 
	
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_LoadSCLFile(IEC61850 clientServer, char * SCLFileName);

/*!	\brief 	Start IEC61850 object communications 
	\ingroup 	Management

	\param 		clientServer		client or Server object to start
	
	\return 	IEC61850_ERROR_NONE on success
	\return 	otherwise error code

	\see IEC61850_Stop()

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
			
		error = IEC61850_Start(myServer); // Starts myServer
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Can't start server: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes  IEC61850_Start(IEC61850 clientServer);

/*! \brief 	Stop IEC61850 object communications
	\ingroup 	Management
	
	\param 	clientServer			client or Server object to stop
	
	\return 	IEC61850_ERROR_NONE on success
	\return 	otherwise error code
	
	\see IEC61850_Start()

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
	
		error = IEC61850_Stop(myServer); // Stops myServer
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Can't stop server: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode
	
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_Stop(IEC61850 clientServer);

/*!  \brief 		Update a given Object of tObjectID with value of ptNewValue
	\ingroup 	DataAttributeAccess

	 \param[in] 	server			Server object to update
	 \param[in] 	ptDAID			Array of IEC61850_DataAttributeID structures. This array identifies the points that are to be updated. This array need to be the same length as the ptNewValue array.
	 \param[in] 	ptNewValue		Array of struct IEC61850_DataAttributeData structures. This holds the new value of the object identified in the ptDAID array. This array need to be the same length as the ptDAID array.
	 \param[in] 	uiCount			The number of IEC61850_DataAttributeID and struct IEC61850_DataAttributeData structures in the array pointed to by ptObjectID and ptNewValue respectably. 

	\return 	IEC61850_ERROR_NONE on success
	\return 	otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		struct IEC61850_DataAttributeData 	Value;
		struct IEC61850_DataAttributeID_Generic  tObjectID;
		Unsigned32		u32Counter;

		u32Counter = 34;

		// Load Data
		Value.ucType =  IEC61850_DATATYPE_INT32;
		Value.uiBitLength = sizeof(u32Counter)*8;
		Value.pvData = &u32Counter;

		// Load ID
		memset(&tObjectID, 0, sizeof(tObjectID));
		tObjectID.Generic_type = IEC61850_DAID_GENERIC;
		tObjectID.uiField1  = 1;
		tObjectID.uiField2 = 6;
		tObjectID.uiField3 = 1;
		tObjectID.uiField4 = 0;
		tObjectID.uiField5 = 0;
	
		error = IEC61850_Update(myServer, (struct IEC61850_DataAttributeID *)&tObjectID, &Value, 1);
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Update failed: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode 

 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_Update(IEC61850 server, struct IEC61850_DataAttributeID * ptDAID, const struct IEC61850_DataAttributeData * ptNewValue, unsigned int uiCount);

/*!  \brief 		Read a value to a given Object ID via the client
	 \ingroup 	DataAttributeAccess
	 
	 \param[in] 	client			Client object to read from
	 \param[in] 	ptDAID			Pointer to EC61850_DataAttributeID structure (or compatable) that idendifies the point that is to be read
	 \param[out] 	ptReturnedValue	Pointer to Object Data structure that hold the returned vaule of the tObjectID
	 
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		struct IEC61850_DataAttributeData 	Value;
		struct IEC61850_DataAttributeID_Generic  tObjectID;
		Unsigned32		u32Counter;

		// Load Data
		Value.ucType =  IEC61850_DATATYPE_INT32;
		Value.uiBitLength = sizeof(u32Counter)*8;
		Value.pvData = &u32Counter;

		// Load ID
		memset(&tObjectID, 0, sizeof(tObjectID));
		tObjectID.Generic_type = IEC61850_DAID_GENERIC;
		tObjectID.uiField1  = 1;
		tObjectID.uiField2 = 6;
		tObjectID.uiField3 = 1;
		tObjectID.uiField4 = 0;
		tObjectID.uiField5 = 0;
		
		error = IEC61850_Read(myServer, (struct IEC61850_DataAttributeID *)&tObjectID, &Value);
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Read failed: %s (%u)", IEC61850_ErrorString(error), error);
		}
		else
		{
			printf("Count = %u",u32Counter);
		}
	\endcode
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_Read(IEC61850 client, struct IEC61850_DataAttributeID * ptDAID, struct IEC61850_DataAttributeData * ptReturnedValue);


/*!  \brief 		Write a value to a given Object ID via the client
	\ingroup 	DataAttributeAccess

	 \param[in] 	client			Client object to write to
	 \param[in] 	ptDAID			Pointer to EC61850_DataAttributeID structure (or compatable) that idendifies the point that is to be written
	 \param[in] 	ptNewValue		Pointer to Object Data structure that hold the new vaule of the tObjectID
	 
	 \return 	IEC61850_ERROR_NONE on success
	 \return 	otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		struct IEC61850_DataAttributeData 	Value;
		struct IEC61850_DataAttributeID_Generic  tObjectID;

		char		bFlag;

		bFlag = 1; // Set flag equal to True

		// Load Data
		Value.ucType =  IEC61850_DATATYPE_BOOLEAN;
		Value.uiBitLength = sizeof(bFlag)*8;
		Value.pvData = &bFlag;

		// Load ID
		memset(&tObjectID, 0, sizeof(tObjectID));
		tObjectID.Generic_type = IEC61850_DAID_GENERIC;
		tObjectID.uiField1  = 1;
		tObjectID.uiField2 = 4;
		tObjectID.uiField3 = 1;
		tObjectID.uiField4 = 0;
		tObjectID.uiField5 = 0;
	
		error = IEC61850_Write(myServer, (struct IEC61850_DataAttributeID *)&tObjectID, &Value);
		if(error != IEC61850_ERROR_NONE)
		{
			printf("Write failed: %s (%u)", IEC61850_ErrorString(error), error);
		}
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_Write(IEC61850 client, struct IEC61850_DataAttributeID * ptDAID, const struct IEC61850_DataAttributeData * ptNewValue);

/*!  \brief 		Gives expliantion of give error code
	 \ingroup 	Support

	 \param[in] 	eErrorCode		Error code in question
	 
	 \return 	pointer to text string explaining error code
	 \return 	otherwise NULL

	Example Usage:
	\code	
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		
		printf("Error: %s", IEC61850_ErrorString(error));
	\endcode 
 */
PIS10_API const char * IEC61850_ErrorString(enum IEC61850_ErrorCodes eErrorCode);

/*!  \brief 	Returns library version number
	 \ingroup 	Support

	 \return 	version number of library  as a string of char with format A.BC.DE.FGH

	Example Usage:
	\code	
		printf("Version number: %s", IEC61850_GetLibraryVersion());
	\endcode 
 */
PIS10_API  const char * IEC61850_GetLibraryVersion(void);

/*!  \brief 	Returns library build time
	 \ingroup 	Support

	 \return 	Build time of the library as a string of char. Format "Mmm dd yyyy hh:mm:ss"

	Example Usage:
	\code	
		printf("Build Time: %s", IEC61850_GetLibraryBuildTime());
	\endcode 
 */
PIS10_API const char * IEC61850_GetLibraryBuildTime(void);


/*!  \brief 		Set Time Quality flags used in UTC timestamp
	\ingroup 		Time
	
	\param[in] 	cLeapSecondKnown					Leap second Flag, Non-zero if the leap second is known, zero if leap second is unknown.
	\param[in] 	cClockFailure							Clock Failure Flag. Non-zero if the clock has failed else zero if clock is functional.
	\param[in] 	cClockNotSynchronized					Clock Not Synchronised Flag. Non-zero if the clock is not synchronized other zero if clock is synchronized.
	\param[in] 	iTimeAccuracyOfFractionsOfSecond		Number of Bits using in faction of seconds part of time stamp (range of 0 to 24). Any value less than 0 mean accuracy is not specified.
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		
		error = IEC61850_SetTimeQuality(1, 0, 1, -1);
		if(error != IEC61850_ERROR_NONE)
		{
			printf("IEC61850_SetTimeQuality has failed: %s", IEC61850_ErrorString(error));
		}
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_SetTimeQuality(char cLeapSecondKnown, char cClockFailure, char cClockNotSynchronized, int iTimeAccuracyOfFractionsOfSecond);

/*!  \brief 		Get Time Quality flags used in UTC Timestamp
	\ingroup 		Time

	\param[out] 	cLeapSecondKnown					Pointer to char which will be set to Non-zero if the leap second is known. May be NULL if not required.
	\param[out] 	cClockFailure							Pointer to char which will be set to Non-zero if the clock has failed. May be NULL if not required.
	\param[out] 	cClockNotSynchronized					Pointer to char which will be set to Non-zero if the clock is not synchronized. May be NULL if not required.
	\param[out] 	iTimeAccuracyOfFractionsOfSecond		Pointer to interger which will be set to the number of bits of time accuracy used in the fraction of seconds part of time stamp (0 to 24 bits). Any number less than 0 mean accuracy is not specified. May be NULL if not required.	
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	Example Usage:
	\code
		enum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
		
		char cLeapSecondKnown = 0;
		char cClockFailure = 0;
		error = IEC61850_GetTimeQuality(&cLeapSecondKnown, &cClockFailure, NULL, NULL);
		if(error != IEC61850_ERROR_NONE)
		{
			printf("IEC61850_GetTimeQuality has failed: %s", IEC61850_ErrorString(error));
		}
		else
		{
			if(cLeapSecondKnown != 0)
			{
				printf(" Leap second known.");
			}
			else
			{
				printf(" Leap second not known.");
			}

			if(cClockFailure != 0)
			{
				printf(" Clock failed. ");
			}
			else
			{
				printf(" Clock OK.");
			}
		}
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_GetTimeQuality(char  * cLeapSecondKnown, char * cClockFailure, char * cClockNotSynchronized, int * iTimeAccuracyOfFractionsOfSecond);

/*!  \brief 		Get the current system time as IEC 61850 Time stamp.
	\ingroup 		Time

	\param[out] 	ptCurrentTime		Pointer to a IEC61850_TimeStamp structure that will be filled with the current time. (With time quality included)
	
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code	

	Example Usage:
	\code

	struct IEC61850_TimeStamp  tTime;
	if(IEC61850_GetTime(&tTime) == IEC61850_ERROR_NONE)
	{
		printf("Seconds: %u Fraction of seconds: %X" tTime.u32Seconds, tTime.u32FractionsOfSecond);
	}
	
		
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_GetTime(struct IEC61850_TimeStamp * ptCurrentTime);

/*!  \brief 		Get Current Time as IEC 61850 Time stamp.
	\ingroup 		Time

	\param[in] 	ptNewTime	New time to set, not including time quality

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code	


	Note: Only works for BECK 124

	Example Usage:
	\code

	struct IEC61850_TimeStamp  tTime;
	
	tTime.u32Seconds = 455896518;
	tTime.u32FractionsOfSecond = 0;
	
	if(IEC61850_SetTime(&tTime) != IEC61850_ERROR_NONE)
	{
		printf("Set Time Faild" );
	}
		
	\endcode 
 */
PIS10_API enum IEC61850_ErrorCodes IEC61850_SetTime(struct IEC61850_TimeStamp * ptNewTime);


/*!  \brief 		Set the user data that will be returned in callback parameters
	 \ingroup 	Support


	 \param[in] 	clientServer		Client/server object to set user data
	 \param[in] 	ptData			Pointer to user data that you want to see when callback called
	 
	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code

	Example Usage:
	\code

	IEC61850 myServer;
	char Data;
	eenum IEC61850_ErrorCodes error = IEC61850_ERROR_NONE;
	
	error = IEC61850_SetUserData(myServer, &Data);
	if(error != IEC61850_ERROR_NONE)
	{
		printf("Write has failed: %s", IEC61850_ErrorString(error));
	}	
	\endcode
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_SetUserData(IEC61850 clientServer,  void * ptData);


/*!	 \brief 		Add/Remove a SNTP Server to the inbuild SNTP client polling table
	 \ingroup 	Time

	 \param[in] 	IPAddress		IP address of SNTP Server you wish to add/remove
	 \param[in] 	uiPollTimems		Polling time in milliseconds. Set to zero if you wish to remove the server

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_SNTPAddServer(char * IPAddress, unsigned int uiPollTimems);

/*!	 \brief 		Get the Data type and size of a given Data Attribute
	 \ingroup 	DataAttributeAccess


	 \param[in] 	clientServer		Client or Server object to search
	 \param[in] 	ptDAID			ID of Data Attribute
	 \param[out] 	ptReturnedType	ucType and uiBitLength will be filled in with the respective type and size of ptDAID. pvData will be null.

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_GetType(IEC61850 clientServer, struct IEC61850_DataAttributeID * ptDAID, struct IEC61850_DataAttributeData * ptReturnedType);


/*!	 \brief 		Select a given control Data object
	 \ingroup 	Control

	 \param[in]	client 							Client object containing control
	 \param[in]	ptControlID 					ID of control that is to be Selected
	 \param[in]	ptSelectValue 					The value the control is to be set to (May be NULL)
	 \param[in] IEC61850_ControlParameters		List of optional parameters including Command Termination timeout, sychro check, and interlock check.

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_ControlSelect(IEC61850 client, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptSelectValue, struct IEC61850_ControlParameters tParam);

/*!	 \brief 		Send an Operate command on given control Data object
	 \ingroup 	Control

	\param[in]	client 							Client object containing control
	\param[in]	ptControlID						ID of control that is to be Operated
	\param[in]	ptOperateValue 					The value the control is to be set to. 
	\param[in] 	IEC61850_ControlParameters		List of optional parameters including Command Termination timeout, sychro check, and interlock check.

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_ControlOperate(IEC61850 client, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptOperateValue, struct IEC61850_ControlParameters tParam);

/*!	 \brief 	Cancel current command on given control Data object
	 \ingroup 	Control
	 \since		V1.36.03

	 \param[in]	client 			Client object containing control
	 \param[in]	ptControlID 	ID of control that is to be canceled
	 \param[in]	ptOperateValue 	The value the control is to be set to.

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API enum IEC61850_ErrorCodes IEC61850_ControlCancel(IEC61850 client, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptCancelValue);


/*!	 \brief 		Print to std out the value of the IEC61850_DataAttributeID structure ptDAID
	 \ingroup 	Support
	 \since		V1.36.03

	 \param[in]	ptDAID	IEC61850_DataAttributeID struct that is to be printed 

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API void IEC61850_PrintDataAttributeID(const struct IEC61850_DataAttributeID * ptDAID);

/*!	 \brief 		Print to std out the value of the IEC61850_DataAttributeData structure ptValue
	 \ingroup 	Support
	 \since		V1.36.03

	 \param[in]	ptValue 	Data value that is to be printed 

	 \return 		IEC61850_ERROR_NONE on success
	 \return 		otherwise error code
*/
PIS10_API void IEC61850_PrintDataAttributeData(const struct IEC61850_DataAttributeData * ptValue);


#ifdef __cplusplus
}
#endif

#endif
