/*****************************************************************************/
/*!	\file 			Pis10Server.cpp
 *	\brief                  PIS10 stack wrapper
 *	\par 			SystemCORP Embedded Technology Pty Ltd
 *
 *				15/50 William St.,
 *				Beckenham,
 *				Perth, WA 6107,
 *				Australia.
 *
 *				Phone	: +61 (8) 9258 4683
 *				Fax	: +61 (8) 9258 4673
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

/******************************************************************************
*	Include
******************************************************************************/
#include "pis10Server.h"
#include <stdio.h>
#include <QtGui/QApplication>
#include <QtGui>
#include "QtServerWidget.h"
#include <time.h>

QtServerWidget * PIS10Server::MyQTwidget = NULL;
PIS10Server * PIS10Server::pThis = NULL;

/*****************************************************************************
* Constructor
******************************************************************************
* Constructor for PIS10 Object.
*
******************************************************************************/
PIS10Server::PIS10Server(QObject *parent) :
    QObject(parent)
{
    MyServer = NULL;
    pThis = this;
}

/*****************************************************************************
* RunServer() Function
******************************************************************************
* Calls create server.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: RunServer()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;

    eError = CreateServer(IEC61850_OPTION_NONE);// should be IEC61850_OPTION_NONE
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("   IEC61850 Client is Created \n");
       // eError = LoadSCLFile("QTDemoClient.cid");
        eError = LoadSCLFile("TqSserver.cid");
        //eError = LoadSCLFile("WINDEMO.ICD");

        if(eError == IEC61850_ERROR_NONE)
        {
            printf("SCL File Loaded\n");
            eError = StartMyServer();
            if(eError == IEC61850_ERROR_NONE)
            {
                printf("Server Started \n");
            }
        }
    }
    return eError;
}

/*****************************************************************************
* RunGUI() Function
******************************************************************************
* Runs GUI.
*
******************************************************************************/
int PIS10Server :: RunGUI(int argc, char *argv[])
{
    QApplication a(argc, argv);

    MyQTwidget = new QtServerWidget();
    MyQTwidget->ptrToPIS10 = this;
    MyQTwidget->show();

    RunServer();

    return a.exec();
}

/*****************************************************************************
* CreateServer() Function
******************************************************************************
* This function sets all the IEC61850 parameters and then calls
* IEC61850_Create API
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: CreateServer(unsigned long int inOptions)
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE ;
    struct IEC61850_Parameters tServerParam ;

    memset(&tServerParam, 0, sizeof(struct IEC61850_Parameters) );

    tServerParam.ClientServerFlag    = IEC61850_SERVER;	// This is a IEC 61850 Server
    tServerParam.uiOptions           = inOptions; // No options set
    tServerParam.ptReadCallback      = (IEC61850_ReadCallback)ReadCallbackHandler; // Assign Read Callback Function
    tServerParam.ptWriteCallback     = (IEC61850_WriteCallback) WriteCallbackHandler; // Assign Write Callback Function
    tServerParam.ptUpdateCallback    = NULL; // SETS Callback Function for Server
    tServerParam.ptSelectCallback    = (IEC61850_ControlSelectCallback) SelectCallbackHandler; // SETS Select commands
    tServerParam.ptOperateCallback   = (IEC61850_ControlOperateCallback) OperateCallbackHandler; // SETS Operate commands
    tServerParam.ptCancelCallback    = (IEC61850_ControlCancelCallback) CancelCallbackHandler; // SETS Cancel commands
    //	tClientParam.uiMaxAssociations	 = 0;

    if(MyServer == NULL)
    {
        MyServer = IEC61850_Create(&tServerParam, &eErrorCode);

        if(MyServer == NULL)
        {
            printf("   Server Failed to create: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
        }
    }
    else
    {
        eErrorCode = IEC61850_ERROR_INVALID_STATE;
    }

    return eErrorCode;
}

/*****************************************************************************
* LoadSCLFile() Function
******************************************************************************
* This function takes the ICD file pointer and then calls
* IEC61850_LoadSCLFile
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: LoadSCLFile(char *pFileName)
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;
    IEC61850 myIEC61850Object = NULL;;

    if(pFileName != NULL)
    {
        myIEC61850Object = GetMyServer();

        eErrorCode = IEC61850_LoadSCLFile(myIEC61850Object, pFileName); // Load in IED configuration SCL file
        if(eErrorCode != IEC61850_ERROR_NONE)
        {
            printf("   Loading SCL file (%s) has failed: %s (%i)\n", pFileName, IEC61850_ErrorString(eErrorCode), eErrorCode);
        }
    }
    else
    {
        eErrorCode = IEC61850_ERROR_INVALID_PARAMETERS;
    }

    return eErrorCode;
}

/*****************************************************************************
* StartMyServer() Function
******************************************************************************
* This function starts IEC61850 Server by calling IEC61850_Start.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: StartMyServer()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Start(GetMyServer()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}

/*****************************************************************************
* StopMyServer() Function
******************************************************************************
* This function stops IEC61850 Server by calling IEC61850_Stop.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: StopMyServer()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Stop(GetMyServer()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}

/*****************************************************************************
* FreeMyServer() Function
******************************************************************************
* This function frees IEC61850 Server by calling IEC61850_Free.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Server :: FreeMyServer()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Free(GetMyServer());  //Frees myServerClient

    if(eErrorCode == IEC61850_ERROR_NONE)
    {
        MyServer = NULL;
    }
    else
    {
        printf("   Free IEC61850 Object has failed: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}


/*****************************************************************************
* GetMyServer() Function
******************************************************************************
* This function returns PIS10 server instance.
*
******************************************************************************/
IEC61850 PIS10Server :: GetMyServer()
{
    return MyServer;
}


/*****************************************************************************
* ReadCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to read an element
* on the server.
******************************************************************************/
//Read Callback to be called by the PIS-10 Stack
int PIS10Server :: ReadCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData *ptReturnedValue)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Read CallBack =======\n");
        printf("     The error code is: %i\n", iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptDataAttributeID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptReturnedValue);
        printf("\n");

        printf("\n     ==============================\n");
        return  iErrorCode;
}


/*****************************************************************************
* WriteCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to write an element
* on the server.
******************************************************************************/
int PIS10Server :: WriteCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData * ptNewValue)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Write CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptDataAttributeID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptNewValue);
        printf("\n");

        printf("\n     ==============================\n");
        return (int) iErrorCode;
}


/*****************************************************************************
* SelectCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to select an control
* element on the server.
******************************************************************************/
int  PIS10Server ::SelectCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptSelectValue, int iSyncroCheck, int iInterlockCheck)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Select CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptSelectValue);
        printf("\n");

        printf("\n     ==============================\n");
        return (int) iErrorCode;
}


/*****************************************************************************
* OperateCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to operate an control
* element on the server.
******************************************************************************/
int  PIS10Server ::OperateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptOperateValue, int iSyncroCheck, int iInterlockCheck)
{
        Integer32 iErrorCode = IEC61850_COMMAND_ERROR_NONE; //Create return value and init to No Error

        struct IEC61850_DataAttributeID_Generic *ptLedControlID=(struct IEC61850_DataAttributeID_Generic *)ptControlID;
        struct IEC61850_DataAttributeData *ptLedOperateValue = (struct IEC61850_DataAttributeData *)ptOperateValue;

        printf("\n\n     ========= Operate CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptOperateValue);
        printf("\n");

        printf("\n     ==============================\n");

        if(ptLedControlID->uiField1 == 2 && ptLedControlID->uiField3 == 1 && ptLedControlID->uiField4 == 0 && ptLedControlID->uiField5 == 0)
        {
            MyQTwidget->LedController(ptLedControlID->uiField2, *(Boolean*)ptLedOperateValue->pvData);
        }
        else if((ptLedControlID->uiField1 == 4 || ptLedControlID->uiField1 == 5)  && ptLedControlID->uiField3 == 1 && ptLedControlID->uiField4 == 0 && ptLedControlID->uiField5 == 0)
        {
            MyQTwidget->XcbrCswController(ptLedControlID->uiField1, *(unsigned char*)ptLedOperateValue->pvData);
        }


        return (int) iErrorCode;
}


/*****************************************************************************
* CancelCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to cancel selection
*  of an control element on the server.
******************************************************************************/
int  PIS10Server ::CancelCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n     ========= Cancel CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     NO DAData for Cancel");
        printf("\n");
        printf("\n     ==============================\n");

        return (int) iErrorCode;
}

/*****************************************************************************
* UpdateCallbackHandler() Function
******************************************************************************
* This function is not used in server side
*
******************************************************************************/
void  PIS10Server ::UpdateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, const struct IEC61850_DataAttributeData *ptNewValue)
{
    Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error
    Unsigned8 u8Value = 0;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;

    IEC61850_DataAttributeID_Generic * DAID = (IEC61850_DataAttributeID_Generic *) ptControlID;
    struct IEC61850_DataAttributeData tDAData;

    tDAData.ucType = IEC61850_DATATYPE_BOOLEAN;
    tDAData.uiBitLength = sizeof(u8Value)*8;
    tDAData.pvData = &u8Value;

    printf("\n     ========= Update CallBack =======\n");
    printf("     The error code is: %i\n",(int) iErrorCode);
    printf("     The DAID is: ");
    IEC61850_PrintDataAttributeID(ptControlID);
    printf("\n     The DAData is: ");
    IEC61850_PrintDataAttributeData(ptNewValue);
    printf("\n");
    printf("\n     ==============================\n");

}
